--
-- PlaceableForestArea
--
-- # Author:  Heady
-- # date: 24.02.2019
-- # last update: 17.03.2019
--
-- > Not for commercial use  < --
-- > Not allowed for edit - Copyright (C) Heady - www.planet-ls.de < --
--

local modDir = g_currentModDirectory;

local modDescXML = loadXMLFile("modDesc", modDir .. "modDesc.xml");
local version = getXMLString(modDescXML, "modDesc.version");

print("Script: PlaceableForestArea v"..tostring(version).." loaded by Heady");


PlaceableForestArea = {};

local PlaceableForestArea_mt = Class(PlaceableForestArea, Placeable);
InitObjectClass(PlaceableForestArea, "PlaceableForestArea");

function PlaceableForestArea:new(isServer, isClient, customMt)
    local mt = customMt;
    if mt == nil then
        mt = PlaceableForestArea_mt;
    end;
    local instance = Placeable:new(isServer, isClient, mt);
    registerObjectClassName(instance, "PlaceableForestArea");

    return instance;
end;

function PlaceableForestArea:delete()
    self.placeableObject:delete();

    --g_farmlandManager:removeStateChangeListener(self);
    unregisterObjectClassName(self);
    PlaceableForestArea:superClass().delete(self);
end;

function PlaceableForestArea:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)
    if not PlaceableForestArea:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
        return false;
    end;
	local xmlFile = loadXMLFile("PlaceableForestAreaXML", xmlFilename);
	
	self.collisionMaskNodes = {};
	if hasXMLProperty(xmlFile, "placeable.collisionMaskNodes") then
        local i = 0;
        while true do
            local key = string.format("placeable.collisionMaskNodes.collisionMaskNode(%d)", i);
            if not hasXMLProperty(xmlFile, key) then
                break;
            end;
			local entry = {};
			entry.node = I3DUtil.indexToObject(self.nodeId, getXMLString(xmlFile, key .. "#node"));
			entry.collisionMask = getXMLInt(xmlFile, key .. "#collisionMask");
			entry.collisionMaskPlacescreen = getXMLInt(xmlFile, key .. "#collisionMaskPlacescreen");
			table.insert(self.collisionMaskNodes, entry);
            i = i + 1;
        end;
	end;
	
	self.visiblePlacementNode = I3DUtil.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.visiblePlacement#node"));
	setVisibility(self.visiblePlacementNode, true);
	
    self.placeableObject = FS19_PlaceableForestArea.ForestArea:new(self.isServer, self.isClient);
    if not self.placeableObject:load(self.nodeId, xmlFile) then
        self.placeableObject:delete();
        delete(xmlFile);
        return false;
    end;
	
	self.placeableObject.finalizePlaced = false;
	--self.placeableObject:register(true); -- add update routine, 
	--table.insert(FS19_BGAone.ModEvent.BGAones, self.placeableObject);
	--self.placeableObject.owningPlaceable = self;
	
	--self:setTipOcclusionAreaDirty();
	
	self.currentScreen = nil;	
	--self.firstPlacement = true;
	self.runOnceUpdate = true;
	
	--print("PlaceableForestArea:load()")

    delete(xmlFile);
    return true;	
end;

function PlaceableForestArea:finalizePlacement()
    PlaceableForestArea:superClass().finalizePlacement(self);
	
    self.placeableObject:register(true); -- add update routine, 
	
	self:setTipOcclusionAreaDirty();

	setVisibility(self.visiblePlacementNode, false);
	for index, entry in pairs(self.collisionMaskNodes) do
		setCollisionMask(entry.node, entry.collisionMask);
	end;
	
	--print("PlaceableForestArea:finalizePlacement()")
	
	self.placeableObject.finalizePlaced = true;
	
	--table.insert(FS19_BGAone.ModEvent.BGAones, self.placeableObject);
	
    --[[if self.isServer then
        self:updateOwnership(true);
    end;]]
    --g_farmlandManager:addStateChangeListener(self);
	
end;

function PlaceableForestArea:readStream(streamId, connection)
    PlaceableForestArea:superClass().readStream(self, streamId, connection);
	
    if connection:getIsServer() then
        local objectId = NetworkUtil.readNodeObjectId(streamId);
        self.placeableObject:readStream(streamId, connection);
        g_client:finishRegisterObject(self.placeableObject, objectId);
    end;

end;

function PlaceableForestArea:writeStream(streamId, connection)
    PlaceableForestArea:superClass().writeStream(self, streamId, connection);
	
    if not connection:getIsServer() then
        NetworkUtil.writeNodeObjectId(streamId, NetworkUtil.getObjectId(self.placeableObject));
        self.placeableObject:writeStream(streamId, connection);
        g_server:registerObjectInStream(connection, self.placeableObject);
    end;
	
end

function PlaceableForestArea:update(dt)
	PlaceableForestArea:superClass().update(self, dt);

	if self.placeableObject.finalizePlaced then
		if self.currentScreen ~= g_gui.currentGui then
			if g_gui.currentGui == g_gui.screens[PlacementScreen] then
				setVisibility(self.visiblePlacementNode, true);

				for index, entry in pairs(self.collisionMaskNodes) do
					setCollisionMask(entry.node, entry.collisionMaskPlacescreen);
				end;
			else
				setVisibility(self.visiblePlacementNode, false);
				
				for index, entry in pairs(self.collisionMaskNodes) do
					setCollisionMask(entry.node, entry.collisionMask);
				end;
			end;
		end;		
	end;
	self.currentScreen = g_gui.currentGui;
	
	if self.runOnceUpdate then
	--print("PlaceableForestArea:update(dt)")
		if self.firstPlacement == nil or self.firstPlacement == true then
			local deform = TerrainDeformation:new(g_currentMission.terrainRootNode)

			for _, levelArea in pairs(self.levelAreas) do -- loop your extra loaded areas
				local layer = -1
				if levelArea.groundType ~= nil then
					layer = g_groundTypeManager:getTerrainLayerByType(levelArea.groundType)
				end
				Placeable.addPlaceableLevelingArea(deform, levelArea, layer, true)
			end

			if self.smoothingGroundType ~= nil then
				deform:setOutsideAreaBrush(g_groundTypeManager:getTerrainLayerByType(self.smoothingGroundType))
			end

			deform:setOutsideAreaConstraints(self.maxSmoothDistance, self.maxSlope, self.maxEdgeAngle)

			deform:setBlockedAreaMaxDisplacement(0.001)
			deform:setDynamicObjectCollisionMask(1048543) -- FFFDF
			deform:setDynamicObjectMaxDisplacement(0.03)

			deform:enablePaintingMode() -- defines that is will only paint and not deform

			-- empty global callback for queueJob (or give callback and callback object to queueJob)
			emptyCallback = function()
			end

			g_terrainDeformationQueue:queueJob(deform, false, "emptyCallback")		
		end;
		self.runOnceUpdate = false;
	end;
	
	self:raiseActive();
end;

function PlaceableForestArea:collectPickObjects(node) 
-- g_currentMission:addNodeObject(triggerId, self); mit triggerId von sub-scripten / Klassen müssen hier behandelt werden. 
-- Alle Shapes ~= NoRigidBody in der i3d werden zu addNodeObject hinzugefügt. Ein weiteres hinzufügen gibt einen Error

    local foundNode = false
    --[[for _, unloadTrigger in ipairs(self.bga.bunker.unloadingStation.unloadTriggers) do
        if node == unloadTrigger.exactFillRootNode then
            foundNode = true
            break
        end
    end
    if not foundNode then
        for _, loadTrigger in ipairs(self.bga.digestateSilo.loadingStation.loadTriggers) do
            if node == loadTrigger.triggerNode then
                foundNode = true
                break
            end
        end
    end]]

    if not foundNode then
        PlaceableForestArea:superClass().collectPickObjects(self, node);
    end;
	
end;

function PlaceableForestArea:loadFromXMLFile(xmlFile, key, resetVehicles)
    if not PlaceableForestArea:superClass().loadFromXMLFile(self, xmlFile, key, resetVehicles) then
        return false;
    end;
	
	self.firstPlacement = Utils.getNoNil(getXMLBool(xmlFile, key.."#firstPlacement"), false);
	--print("PlaceableForestArea:loadFromXMLFile()")
    self.placeableObject:loadFromXMLFile(xmlFile, key, resetVehicles);

    return true;	
end;

function PlaceableForestArea:saveToXMLFile(xmlFile, key, usedModNames)
    PlaceableForestArea:superClass().saveToXMLFile(self, xmlFile, key, usedModNames)
	
    self.placeableObject:saveToXMLFile(xmlFile, key, usedModNames);
	
end;

function PlaceableForestArea:setOwnerFarmId(ownerFarmId, noEventSend)
    PlaceableForestArea:superClass().setOwnerFarmId(self, ownerFarmId, noEventSend);

    if self.placeableObject ~= nil then
        self.placeableObject:setOwnerFarmId(ownerFarmId);
    end;
end;


